#!/usr/bin/perl

use strict;
use MSACS::syntree;


my $help = << "end_of_help;";
Usage:   trees in the standard CESS-ECE format > tree2column.pl [options] > trees in Start-End
Options: 
     -w [0|1]     Output a column of words (default 0).
     -p [0|1]     Output a column of PoS tags (default 0).
     -l [0|1]     Output a column of lemas (default 0).
     -sc [0|1]    Output a column of semantic classes (default 0).
     -s [0|1]     Output a column of senses (default 0).
     -tn [0|1]    Output a column of target nouns (default 0).
     -tv [0|1]    Output a column of target verbs (default 0).
     -sy [0|1]    Output a column of syntactic analysis (default 0).
     -ne [0|1]    Output a column of named entities (default 0).
     -pr [0|1]    Output the colums of the propositions (default 0).

     -BII [0|1]   Output the Basic Input Info annotation level (default 0).
     -EII [0|1]   Output the Extra Input Info annotation level (default 0).
     -NER [0|1]   Output the annotation level for NER task (default 0).
     -NSD [0|1]   Output the annotation level for NSD task (default 0).
     -SRL [0|1]   Output the annotation level for SRL task (default 0).

     -ALL [0|1]   Output all the columns (default 0).

end_of_help;

my $words = 0;
my $pos = 0;

my $lemai = 0;
my $semclassi = 0;
my $sensei = 0;
my $targetNouni = 0;
my $targetVerbi = 0;
my $syntaxi = 0;
my $nei = 0;
my $propsi = 0;
my $BIIi = 0;
my $EIIi =0;
my $NERi = 0;
my $NSDi = 0;
my $SRLi = 0;
my $ALLi = 0;

while (@ARGV) {
    my $a = shift @ARGV; 

    if ($a eq "-w") {
	$words = shift @ARGV; 
    }
    elsif ($a eq "-p") {
	$pos = shift @ARGV; 
    }
    elsif ($a eq "-l") {
	$lemai = shift @ARGV; 
    }
    elsif ($a eq "-sc") {
	$semclassi = shift @ARGV; 
    }
    elsif ($a eq "-s") {
	$sensei = shift @ARGV; 
    }
    elsif ($a eq "-tn") {
	$targetNouni = shift @ARGV; 
    }
    elsif ($a eq "-tv") {
	$targetVerbi = shift @ARGV; 
    }
    elsif ($a eq "-sy") {
	$syntaxi = shift @ARGV; 
    }
    elsif ($a eq "-ne") {
	$nei = shift @ARGV; 
    }
    elsif ($a eq "-pr") {
        $propsi = shift @ARGV; 
    }
    elsif ($a eq "-BII") {
        $BIIi = shift @ARGV; if ($BIIi){  $words = 1; $targetNouni = 1; $targetVerbi = 1;  }
    }
    elsif ($a eq "-EII") {
        $EIIi = shift @ARGV; if ($EIIi){  $lemai = 1; $pos = 1; $syntaxi = 1;  }
    }
    elsif ($a eq "-NER") {
        $NERi = shift @ARGV; if ($NERi){  $nei = 1;  }
    }
    elsif ($a eq "-NSD") {
        $NSDi = shift @ARGV; if ($NSDi){  $sensei = 1;  }
    }
    elsif ($a eq "-SRL") {
        $SRLi = shift @ARGV; if ($SRLi){  $semclassi = 1; $propsi = 1;  }
    }
    elsif ($a eq "-ALL") {
        $ALLi = shift @ARGV; if ($ALLi){  $words = 1; $targetNouni = 1; $targetVerbi = 1; $lemai = 1; $pos = 1; $syntaxi = 1; $nei = 1; $sensei = 1; $semclassi = 1; $propsi = 1;}
    }
    else {	

    }
}

my $st = MSACS::syntree->read_wsj_mrg(\*STDIN);

while ( defined $st ) {
#    my @tags = $st->to_SE_tagging;
#    print join("\n", @tags), "\n\n";

    my ($n,@S,@E,@W,@P,@LEMA,@SEMCLASS,@SENSE, @TARGETNOUN, @TARGETVERB, @NEs, @NEe, @PROPS); #information columns to be showed
    my $countProps = 0;   #PROPS counter
    my %sentenceEndPoint;

    ####To treat j-CC-ArgM inside S.co
    my $clauseIndex = 0;    #Real clause counter. Two sibling clauses have the same clauseIndex and the same father.
    my @propForLostArgs;    #Contains start, end and content of every arg in the sentence, to be able to recover lost args with coord as siblings.
    my %clauseIndexEndPoint;
    my %coordinatedClause;
    my %verbsInClause;
    my @lostArgM;
    ####

    foreach $n ( $st->dfs) {
	if ($n->is_terminal) {

	    my $w = $n->content;

	    my $pos = parsePOS($w->pos);
	    $P[$w->id] = $pos;

	    $W[$w->id] = $w->form;
	    $W[$w->id] =~ s/^-Fpa-$/(/;
	    $W[$w->id] =~ s/^-Fpt-$/)/;

	    $LEMA[$w->id] = $w->lema;
	    $LEMA[$w->id] =~ s/^-Fpa-$/(/;
	    $LEMA[$w->id] =~ s/^-Fpt-$/)/;

	    $SEMCLASS[$w->id] = $w->semclass;

	    $SENSE[$w->id] = $w->sense;
	    $SENSE[$w->id] =~ s/^C[0-9]S$/CS/;

	    #Derive targetNoun info
	    $TARGETNOUN[$w->id] = ( ($w->sense =~ /^[0-9]{8}n$/) || ($w->sense =~ /^CS$/) ) ? "*" : "-";
	    #Derive targetVerb info
	    $TARGETVERB[$w->id] = ($SEMCLASS[$w->id]) ? "*" : "-";

	    #Verb for the SRL
	    if ($SEMCLASS[$w->id]){

	      $PROPS[$countProps]{'S'}[$w->id] .= "(V";
	      $PROPS[$countProps]{'E'}[$w->id] = ")".$PROPS[$countProps]{'E'}[$w->id];
	      $PROPS[$countProps]{'hasAverb'} = 1;      # Some tags like S.NF.C.co don't have a verb so we mark that
	      $PROPS[$countProps]{'verbI'} = $w->id;    # it doesn't have a verb to avoid printing a PROP column for it.

	      ####To treat j-CC-ArgM inside S.co
	      $verbsInClause{$clauseIndex}{$countProps} = 1; #The first index is for clauses, the second for PROPS.
	      ####
	    }
	    #It is a NE
	    if (isaNEtag($pos)){
	      my $nerType = parseTermNE($pos);
	      #if ($NEs[$w->id] !~ /$nerType$/){
	      if (!booked(\@NEs, \@NEe, $w->id, $nerType)){
		# If the NE tag is already set (due to the POS and the syntagmatic labelling) we do not replicate it.
		$NEs[$w->id] .= "(".$nerType;
		$NEe[$w->id] = ")".$NEe[$w->id];
	      }
	    }

	    if (defined($sentenceEndPoint{$w->id})){
	      closeProps(\@PROPS, $countProps, $sentenceEndPoint{$w->id});
	      $countProps = $countProps - $sentenceEndPoint{$w->id};
	      undef($sentenceEndPoint{$w->id});
	    }

	    ####To treat j-CC-ArgM inside S.co
	    if (defined($clauseIndexEndPoint{$w->id})){
	      $clauseIndex = $clauseIndex - $clauseIndexEndPoint{$w->id};
	      undef($clauseIndexEndPoint{$w->id});
	    }
	    ####
	}
	else {
	    my $p = $n->content;
	    my $pStart = $p->start;
	    my $pEnd = $p->end;
	    if ($pStart == $pEnd + 1) { $pEnd = $pStart;  }
	
	    my $oldCountProps = $countProps;
	    my $oldClauseIndex = $clauseIndex;
	    #If the node starts a sentence, we store its end point and create a PROPS column.
	    if (isaSentence($p->type)){
	      $sentenceEndPoint{$pEnd}++;
	      $countProps++; #Create a PROPS column

	      ####To treat j-CC-ArgM inside S.co
	      $clauseIndexEndPoint{$pEnd}++;
	      $clauseIndex++;
	      if ($p->type =~ /S\.co/){ $coordinatedClause{$clauseIndex} = 1; }
	      ####

	      if ($PROPS[$countProps]{'closed'} == 1) {                 # If the proposed index for the new sentence is already closed
		my $offset = getOffsetToNextFree(\@PROPS, $countProps); # then   make index jump x extra places to the next free one
		$countProps += $offset;                                 # and    when jumping back, jump x extra places instead of one
		$sentenceEndPoint{$pEnd} += $offset;
	      }
	      if (isACoord($p->type)) {  $PROPS[$oldCountProps]{'hasCoordAsSibling'} = $clauseIndex;  } #To treat lost args with coo as sibling.
	    }

	    if (isaNEtag($p->type)){ #It is a NE
	      my $nerType = parseNonTermNE($p->type);
	      $NEs[$pStart] .= "(".$nerType;
	      $NEe[$pEnd] = ")".$NEe[$pEnd];
	    }

	    if($p->type){

	      if (isaClauseWithEllipticVerb($p->type)) {  $PROPS[$countProps]{'hasEllipticVerb'} = 1;  } #Tag it to be discarded for orphan arg report.

	      if ($p->type =~ /^(.+)\-(Arg.*)$/){ #It is a syntactic constituent plus a semantic role.

		#Syntactic part
     		$S[$pStart] .= "(".$1;
		$E[$pEnd] = ")".$E[$pEnd];

		#SRL part:
		my $iProps;
		my $iClause;
		if (isaSentence($p->type)){ #If the role is attached to the clause start, it belongs to the previous clause
		  $iProps = $oldCountProps;
		  $iClause = $oldClauseIndex;
		}
		else{
		  $iProps = $countProps;
		  $iClause = $clauseIndex;
		}

		$PROPS[$iProps]{'S'}[$pStart] .= "(".$2;
		$PROPS[$iProps]{'E'}[$pEnd] = ")".$PROPS[$iProps]{'E'}[$pEnd];

		###To treat lost args with coord as sibling
		push(@{$propForLostArgs[$iProps]}, $pStart);
		push(@{$propForLostArgs[$iProps]}, $pEnd);
		push(@{$propForLostArgs[$iProps]}, $2);
		###
		
		my $ArgType = $2;
		if ($ArgType =~ /Arg[0-9]/){ $PROPS[$iProps]{'hasNumArgs'} = 1; } 
		if ($ArgType =~ /Arg[ML]/){ $PROPS[$iProps]{'hasAdvArgs'} = 1; } 
	
		####To treat j-CC-ArgM inside S.co
		if ( ($p->type =~ /^(.+)j\-CC\-(ArgM.*)$/) && ($coordinatedClause{$iClause}) ){
		  push(@lostArgM, $pStart);
		  push(@lostArgM, $pEnd);
		  push(@lostArgM, $2);
		  push(@lostArgM, $iClause);
		  push(@lostArgM, $iProps);
		  push(@lostArgM, "NORMAL"); #push(@lostArgM, "AVOIDNEXTONE");
		}
		####

	      }
	      else{ #It is just a syntactic constituent
		$S[$pStart] .= "(".$p->type;
		$E[$pEnd] = ")".$E[$pEnd];
	      }
	    }


#	    if ($p->type) {
#		$S[$pStart] .= "(".$p->type;
#		$E[$pEnd] = ")".$E[$pEnd]; 
#	        #$E[$pEnd] = $p->type.")".$E[$pEnd];
#	    }
	}
    }

    ####To treat j-CC-ArgM inside S.co
    if (@lostArgM){  modifyProps(\@lostArgM, \@PROPS, \%verbsInClause);  }
    ####

    ###Delete args embedding verbs
    delArgsEmbeddingVerbs(\@PROPS, \@propForLostArgs);
    ###

    ####To treat orphan args with coordinated sentences as siblings
    treatOrphanWithCoordAsSibling(\@PROPS, \@propForLostArgs, \%verbsInClause);
    ###

    ####Follow textual order
    textualOrder(\@PROPS);
    ###

    my $i;
    for ($i=0; $i<@W; $i++) {
	if ($words) {
	
	    $W[$i] =~ s/-LRB-/(/g;
	    $W[$i] =~ s/-RRB-/)/g;
	    $W[$i] =~ s/-LCB-/{/g;
	    $W[$i] =~ s/-RCB-/}/g;

	    printf("%-18s  ", $W[$i]);
	}

	if ($targetNouni) {  printf("%-1s  ", $TARGETNOUN[$i]);  }
	if ($targetVerbi) {  printf("%-1s  ", $TARGETVERB[$i]);  }
	if ($lemai) {  printf("%-18s  ", $LEMA[$i]);  }
	
	if ($pos) {

	    $P[$i] =~ s/-LRB-/(/g;
	    $P[$i] =~ s/-RRB-/)/g;
	    $P[$i] =~ s/np0000[opla]/np0000/g;

	    printf("%-8s  ", $P[$i]);
	}

	if ($syntaxi) {

	  $S[$i] =~ s/sn[opland]/sn/g;
	  printf("%28s*%-13s  ", $S[$i], $E[$i]);
	}

	if ($nei) {  printf("%8s*%-4s  ", $NEs[$i], $NEe[$i]);  }

	
	if ($sensei) {  ($SENSE[$i]) ? printf("%-9s  ", $SENSE[$i]) : printf("%-9s  ", "-");  }

	if ($semclassi) {  ($SEMCLASS[$i]) ? printf("%-3s ", $SEMCLASS[$i]) : printf("%-2s  ", "-");  }

	if($propsi) {
	  for (my $j = 1; $j < scalar(@PROPS); $j++){
	    if ($PROPS[$j]{'hasAverb'} == 1){
	      if ($j != 1){ printf("  ");  }
	      printf("%11s*%-6s", $PROPS[$j]{'S'}[$i], $PROPS[$j]{'E'}[$i]);
	    }
	  }
	}
	
	printf("\n")	

    }
    print "\n";

    $st = MSACS::syntree->read_wsj_mrg(\*STDIN);

}


sub treatOrphanWithCoordAsSibling{ #Recovers Args which don't have a verb as a sibling but have a coordinated clause as a sibling.
  my ($refVectorProps, $refPropForLostArgs, $refHashVerbsInClause) = @_;

  my @props = @{$refVectorProps};
  my @lostArgsProp = @{$refPropForLostArgs};
  for (my $j = 1; $j < scalar(@props); $j++){
    if ( ($props[$j]{'hasAverb'} != 1) && ($props[$j]{'hasEllipticVerb'} != 1)
	 && ( ($props[$j]{'hasNumArgs'} == 1) || ($props[$j]{'hasAdvArgs'} == 1) )){
      if (defined($props[$j]{'hasCoordAsSibling'})) {
	#We have orphans which must be recovered
	while (@{$lostArgsProp[$j]}){
	  my $start = shift(@{$lostArgsProp[$j]});
	  my $end = shift(@{$lostArgsProp[$j]});
	  my $content = shift(@{$lostArgsProp[$j]});
	  if ($content =~ /Arg[0-9]/){ #Until here we could only have recovered AdvArgs for this prop.
	    my $nextClauseWithVerb = findNextClauseWithVerb($props[$j]{'hasCoordAsSibling'}, $refHashVerbsInClause);
	    if ($nextClauseWithVerb != -1){
	      copyArgToEveryPropInClause($start, $end, $content, "NORMAL", $nextClauseWithVerb, $refHashVerbsInClause, $refVectorProps);
	      $refVectorProps->[$j]{'recoveredNumArgs'} = 1;#Mark as recov to avoid printing in the orphanArgs report.
	    }else{ }#orphan arg
	  }
	  else{
	    if ($props[$j]{'recoveredAdvArgs'} != 1){ #Make sure that modifyProps has not already recovered this AdvArgs
		my $nextClauseWithVerb = findNextClauseWithVerb($props[$j]{'hasCoordAsSibling'}, $refHashVerbsInClause);
		if ($nextClauseWithVerb != -1){
		  copyArgToEveryPropInClause($start, $end, $content, "NORMAL", $nextClauseWithVerb, $refHashVerbsInClause, $refVectorProps);
		  $refVectorProps->[$j]{'recoveredAdvArgs'} = 1;#Mark as recov to avoid printing it in the orphanArgs report.
		}else{ } #orphan arg
	    }
	  }
	}
      } #end of        if (defined($props[$j]{'hasCoordAsSibling'})) {
      else{  }
    }
  }
}

sub modifyProps{
my ($refArgM, $refProps, $refHashVerbsInClause) = @_;

  while (@{$refArgM}){
    my $start = shift(@{$refArgM});
    my $end = shift(@{$refArgM});
    my $content = shift(@{$refArgM});
    my $clauseI  = shift(@{$refArgM});
    my $propsI = shift(@{$refArgM});
    my $exception = shift(@{$refArgM});

    if ($refProps->[$propsI]{'hasEllipticVerb'} != 1){
      my $nextClauseWithVerb = findNextClauseWithVerb($clauseI, $refHashVerbsInClause);
      if ($nextClauseWithVerb != -1){
	copyArgToEveryPropInClause($start, $end, $content, $exception, $nextClauseWithVerb, $refHashVerbsInClause, $refProps);
	$refProps->[$propsI]{'recoveredAdvArgs'} = 1; #Mark it as recovered to avoid printing it in the orphan args report.
      }
      else{  }
    }
  }
}

sub copyArgToEveryPropInClause{
my ($start, $end, $content, $exception, $nextClauseWithVerb, $refHashVerbsInClause, $refProps) = @_;

  my @props = @{$refProps};

  my %internHash = %{$refHashVerbsInClause};

  foreach my $element (sort{ $a <=> $b }(keys(%{$internHash{$nextClauseWithVerb}}))){
    if ($exception eq "AVOIDNEXTONE"){  $exception = "NORMAL"; }
    else{
      # If the arg does not embed the verb, we copy it.
      my $verbIndex = findVerb(\@{$props[$element]{'S'}});
      if ($verbIndex != -1){
	if ( ($start <= $verbIndex) && ($verbIndex <= $end) ){ }
	else{
	  $refProps->[$element]{'S'}[$start] .= "(".$content;
	  $refProps->[$element]{'E'}[$end] = ")".$refProps->[$element]{'E'}[$end];
	}
      }
    }
  }

}

sub findNextClauseWithVerb{
my ($clauseI, $refHashVerbsInClause) = @_;

  my %internHash = %{$refHashVerbsInClause};
  foreach my $element (sort{ $a <=> $b }(keys(%internHash))){
    if ($element > $clauseI){
      return($element);
    }
  }
  return(-1);
}

sub delArgsEmbeddingVerbs{
  my ($refPROPS, $refPropForLostArgs) = @_;

  my @props = @{$refPROPS};
  my @propForLostArgs = @{$refPropForLostArgs};
  for (my $j = 1; $j < scalar(@props); $j++){
    if ($props[$j]{'hasAverb'}){
      my $verbIndex = findVerb(\@{$props[$j]{'S'}});
      if ($verbIndex != -1){
	if (defined($propForLostArgs[$j])){
	  for (my $i=0; $i < scalar(@{$propForLostArgs[$j]}); $i = $i + 3) {
	    if( ($propForLostArgs[$j][$i] <= $verbIndex) && ($verbIndex <= $propForLostArgs[$j][$i+1]) ){ #Verb is embedded in the Arg
	      deleteArg($refPROPS, $refPropForLostArgs, $j, $i);
	      $i = $i - 3; #We have extracted an arg ($start, $end, $content) from the array
	    }
	  }
        }
      }
    }
  }
}

sub deleteArg{
  my ($refVectorProps, $refPropForLostArgs, $j, $i) = @_;

  my $start = $refPropForLostArgs->[$j][$i];
  my $end = $refPropForLostArgs->[$j][$i+1];
  my $content = $refPropForLostArgs->[$j][$i+2];

  @{$refPropForLostArgs->[$j]} = splice(@{$refPropForLostArgs->[$j]}, $i, $i+1, $i+2); #Delete the args

  if ($refVectorProps->[$j]{'S'}[$start] =~ /\($content/){
    $refVectorProps->[$j]{'S'}[$start] =~ s/\($content//; #Remove content
  }

  if ($refVectorProps->[$j]{'E'}[$end] =~ /\)$/) {
    chop($refVectorProps->[$j]{'E'}[$end]);  } #Remove end mark

}

sub findVerb{
  my ($refStartVector) = @_;

  my @vect = @{$refStartVector};

  my $j = 0;
  while ( ($vect[$j] !~ /\(V/) && ($j < scalar(@vect)) ){  $j++;  }

  if ($vect[$j] =~ /\(V/){  return ($j);  }
  else{  return(-1);  }
}

sub copyContent{
my ($refPROPS, $propsI, $refHash, $hashI, $mode) = @_;

my @labels = ('S', 'E', 'hasAverb', 'hasEllipticVerb', 'hasNumArgs', 'recoveredNumArgs', 'hasAdvArgs', 'recoveredAdvArgs', 'recovered');

  foreach my $element (@labels){
    if ($mode eq "prop2hash"){  $refHash->{$hashI}{$element} = $refPROPS->[$propsI]{$element};  }
    elsif ($mode eq "hash2prop"){  $refPROPS->[$propsI]{$element} = $refHash->{$hashI}{$element};  }
  }

  if ($mode eq "prop2hash"){  $refHash->{$hashI}{'reportIndex'} = $propsI; } #We keep the index of the prop as it is in the tree file for the report.
  elsif ($mode eq "hash2prop"){  $refPROPS->[$propsI]{'reportIndex'} = $refHash->{$hashI}{'reportIndex'};  }

}

sub textualOrder{
my ($refPROPS) = @_;

my %wVerb;
my %woVerb;
my $keyForVerblessProps = 1;

   for (my $j = 1; $j < scalar(@{$refPROPS}); $j++){
     if (defined($refPROPS->[$j]{'hasAverb'})){
       copyContent($refPROPS, $j, \%wVerb, $refPROPS->[$j]{'verbI'}, "prop2hash");
     }
     else{
       copyContent($refPROPS, $j, \%woVerb, $keyForVerblessProps, "prop2hash");
       $keyForVerblessProps++;
     }
   }

   my $i = 1;
   foreach my $element (sort{ $a <=> $b }(keys(%wVerb)) ) {
     copyContent($refPROPS, $i, \%wVerb, $element, "hash2prop");
     $i++;
   }
   foreach my $element (keys(%woVerb)){
     copyContent($refPROPS, $i, \%woVerb, $element, "hash2prop");
     $i++;
   }


  if ($i != scalar(@{$refPROPS}) ) {  die "Inconsistency when reordering columns.\n";  }
}

sub closeProps{
my ($refProps, $max, $decr) = @_;

  for (my $i = $max - $decr +1; $i <= $max; $i++){
    $refProps->[$i]{'closed'} = 1;
  }
}

sub booked{ #is this named entity already in this position?
my ($refStartV, $refEndV, $position, $nerType) = @_; #exact means same start, end and type
  if (($refStartV->[$position] =~ /$nerType$/) && ($refEndV->[$position] =~ /^\)/) ){
    return(1);
  }
  else{  return(0);  }
}

sub isaSentence{
my ($item) = @_;

  if ( ($item =~ /^S\./) || ($item eq "S") || ($item =~ /^S\*/) ){  return (1);  }
  else{  return (0);  }

}

sub isaSentenceWithVerb{ # A sentence with V is interesting for the PROPS column
my ($item) = @_;         # If there is no V in the clause, the args should'nt be taken into account for the PROPS column

  if ( ($item =~ /^S\./) || ($item eq "S") ){  return (1);  } #|| ($item =~ /^S\*/)
  else{  return (0);  }
}

sub isaClauseWithEllipticVerb{ # This function tells the system which 'S' should discard when looking for orphan args.
my ($item) = @_;

  if ( ($item =~ /^S\*/) || ($item =~ /^S\.F\.[ACR]\*/) || ($item =~ /^S\.F\.A(Comp|Conc|Cond|Cons)\*/) || ($item =~ /^S\.NF\.[ACP]\*/)
    || ($item =~ /^S\.NF\.C\.co/)){ #The first row detects elliptic verbs. And S.NF.C.co does not contain directly any verb.
    return (1);
  }
  else{  return (0);  }
}

sub isACoord{
my ($item) = @_;

  if ($item =~ /^S.*\.co/){  return (1);  } #|| ($item =~ /^S\*/)
  else{  return (0);  }

}

sub isaNEtag{
my ($item) = @_;

  if ( ($item eq "np0000o")
    || ($item eq "np0000p")
    || ($item eq "np0000l")
    || ($item eq "np0000a")
    || ($item eq "W")
    || ($item eq "Z")
    || ($item eq "Zp")
    || ($item eq "Zm")
    || ($item =~ /^sno/)
    || ($item =~ /^snp/)
    || ($item =~ /^snl/)
    || ($item =~ /^sna/)
    || ($item =~ /^snn/)
    || ($item =~ /^snd/)
    ) {  return (1);  }
  else{  return (0);  }
}

sub getOffsetToNextFree{
 my ($refVector, $i) = @_;

  my $j = 0;
  while ($refVector->[$i]{'closed'} == 1){
    $i++;
    $j++;
  }
  return($j);
}

sub parsePOS{
my ($param) = @_;

  $param =~ s/^-LRB-$/(/;
  $param =~ s/^-RRB-$/)/;

  return($param);
}

sub parseNonTermNE{
my ($param) = @_;

  if ($param =~ /^snd/) {  return("DAT");  }
  if ($param =~ /^snn/) {  return("NUM");  }
  if ($param =~ /^snp/) {  return("PER");  }
  if ($param =~ /^snl/) {  return("LOC");  }
  if ($param =~ /^sno/) {  return("ORG");  }
  if ($param =~ /^sna/) {  return("OTH");  }

return($param);
}

sub parseTermNE{
my ($param) = @_;

  $param =~ s/^np0000o$/ORG/;
  $param =~ s/^np0000p$/PER/;
  $param =~ s/^np0000l$/LOC/;
  $param =~ s/^np0000a$/OTH/;
  $param =~ s/^Z$/NUM/;
  $param =~ s/^Zp$/NUM/;
  $param =~ s/^Zm$/NUM/;
  $param =~ s/^W$/DAT/;


  return($param);
}
