##################################################################
#
#  Package   w o r d  :  a word
#
#  January 2007
#
#  A word, containing id (position in sentence), form, PoS, lema, semantic class and sense tag
#
##################################################################

use strict; 

package MSACS::word; 

# Constructor: creates a new word
# Parameters: id (position), form, PoS, lema, semclass and sense tags.
# targetNouns and targetVerbs fields are derived from sense tag.
sub new {
    my ($pkg, @fields) = @_;

    my $w = []; 

    $w->[0] = shift @fields;  # id (position in sentence)
    $w->[1] = shift @fields;  # form
    $w->[2] = shift @fields;  # PoS

    my $lema = shift @fields; 
    if ($lema =~ /^(.*)\-([a-z|A-Z][0-9])$/){  #lema: 'cautivar-c2' o 'cautivar'
      $w->[3] = $1;  # lema 
      $w->[4] = $2;  # semclass 
    }
    else{
      $w->[3] = $lema;  # lema 
      $w->[4] = undef;  # semclass 
    }
    $w->[5] = shift @fields;  # sense 


    return bless $w, $pkg; 
}

# returns the id of the word
sub id {
    my $w = shift;
    return $w->[0];
}

# returns the form of the word
sub form {
    my $w = shift;
    return $w->[1];
}

# returns the PoS tag of the word
sub pos {
    my $w = shift;
    return $w->[2];
}

# returns the lema of the word
sub lema { 
    my $w = shift;
    return $w->[3];
}

# returns the semantic class of the word
sub semclass { 
    my $w = shift;
    return $w->[4];
}

# returns the sense of the word
sub sense { 
    my $w = shift;
    return $w->[5];
}


sub to_string {
    my $w = shift; 
    return "w@".$w->[0].":".$w->[1].":".$w->[2].":".$w->[3].":".$w->[4].":".$w->[5]; 
}

1;





